'use strict';
function ImageManager(MPEditor) {
  this.MPEditor = MPEditor;
  this.images = {};
  this.descFocused = false;
  this.imageContextConfig = {
    0: {
      blotName: 'image-uploading',
      tagName: 'div',
      className: 'plugin-image-uploading',
      template: `<span>图片上传中…</span>`.replace(/>\s+</g, '><'),
    },
    1: {
      blotName: 'image-fail',
      tagName: 'div',
      className: 'plugin-image-fail',
      template: `<p class="plugin-image-fail-icon"></p>
      <p class="plugin-image-fail-title">图片上传失败</p>
      <p class="plugin-image-fail-text">{{errorMessage}}</p>
      <p class="plugin-image-fail-buttons">
        <span style="display:{{display}};" class="plugin-image-fail-button" data-type='retry'>重试</span>
        <span class="plugin-image-fail-button" data-type='delete'>删除</span>
      </p>`.replace(/>\s+</g, '><'),
    },
    2: {
      blotName: 'image-success',
      tagName: 'div',
      className: 'plugin-image-success',
      template: `<img src="{{url}}" class="plugin-image-success-image"/>
      <p class="plugin-image-success-operates">
        <span class="img-desc" contenteditable="true" data-desc="{{desc}}">{{desc}}</span>
        <span class="plugin-image-success-count" contenteditable="false">0/60</span>
        <span class="plugin-image-success-buttons">
          <span class="plugin-image-success-button" data-type='edit'>编辑图片</span>
          <span class="plugin-image-success-button" data-type='delete'>删除</span>
        </span>
      </p>`.replace(/>\s+</g, '><'),
    },
  };

  // 初始化

  var imageManager = this;
  var compositing = false;

  function countDesc($span) {
    var desc = $span.text();
    var length = MPEditor.computeWordCount(desc).length;
    $span
      .next()
      .css('color', length > 60 ? '#ff4242' : '#a3a5b3')
      .text(`${length}/60`);
  }

  // 上传成功图片操作按钮显示或隐藏
  $('#editor').delegate('.plugin-image-success-image', 'click', function (
    event
  ) {
    var $image = $(event.currentTarget);
    var $wrapper = $image.parent();
    var $buttons = $wrapper.find('.plugin-image-success-buttons');
    var display = $buttons.css('display');
    if (display === 'none') {
      imageManager.hideOperates();
      $buttons.css('display', 'flex');
      $buttons[0].scrollIntoViewIfNeeded(false);
      quill.blur();
    } else {
      imageManager.hideOperates();
      quill.blur();
    }
  });

  // 上传成功图片操作按钮点击
  $('#editor').delegate('.plugin-image-success-buttons', 'click', function (
    event
  ) {
    var $buttons = $(event.currentTarget);
    var $wrapper = $buttons.parent().parent();
    var token = $wrapper.data('token');
    var type = $(event.target).data('type');
    var imageContext = imageManager.images[token].imageContext;
    var $node = imageManager.images[token].$node;
    if (imageContext && type) {
      if (type === 'edit') {
        imageContext.edit();
      }
      if (type === 'delete') {
        imageContext.delete();
        $node.remove();
      }
      imageManager.hideOperates();
    }
  });

  // 上传失败图片操作按钮点击
  $('#editor').delegate('.plugin-image-fail-buttons', 'click', function (
    event
  ) {
    var $wrapper = $(event.currentTarget).parent();
    var token = $wrapper.data('token');
    var type = $(event.target).data('type');
    var $node = imageManager.images[token].$node;
    var imageContext = imageManager.images[token].imageContext;
    if (imageContext && type) {
      if (type === 'retry') {
        imageContext.retry();
      }
      if (type === 'delete') {
        imageContext.delete();
        $node.remove();
      }
    }
  });

  $('#editor')
    .delegate('.img-desc', 'focus', function (event) {
      imageManager.descFocused = true;
      imageManager.hideOperates();
      var $span = $(event.currentTarget);
      var desc = $span.text();
      if (!desc.trim()) {
        $span.empty();
      }
      countDesc($span);
      SHEditor.triggerTextTagChange(
        Object.myAssign(MPEditor.correctedTagStatus, { disabled: true })
      );
    })
    .delegate('.img-desc', 'compositionstart', function (event) {
      compositing = true;
    })
    .delegate('.img-desc', 'input', function (event) {
      if (!compositing) {
        var $span = $(event.currentTarget);
        countDesc($span);
      }
    })
    .delegate('.img-desc', 'compositionend', function (event) {
      compositing = false;
      var $span = $(event.currentTarget);
      countDesc($span);
    })
    .delegate('.img-desc', 'paste', function (event) {
      var span = event.currentTarget;
      var $span = $(span);
      var content = span.data;
      content = content && content.replace(/[\n\r]/g, '');
      var selection = window.getSelection();
      var focusOffset = selection.focusOffset;
      var currentOffset = content.length + focusOffset;
      var range = selection.getRangeAt(0);
      range.deleteContents();
      var node = document.createTextNode(content);
      range.insertNode(node);
      $span.text($span.text());
      range.setStart($span[0].childNodes[0], currentOffset);
      range.setEnd($span[0].childNodes[0], currentOffset);
      range.collapse(true);
      selection.removeAllRanges();
      selection.addRange(range);
      countDesc($span);
    })
    .delegate('.img-desc', 'blur', function (event) {
      var $span = $(event.currentTarget);
      var desc = $span.text();
      var string = MPEditor.computeWordCount(desc, 60).string;
      var length = MPEditor.computeWordCount(desc, 60).length;
      $span.text(string);
      $span.attr('data-desc', string);
      countDesc($span);
      SHEditor.triggerTextTagChange(
        Object.myAssign(MPEditor.correctedTagStatus, { disabled: false })
      );
      imageManager.descFocused = false;
    })
    .delegate('.img-desc', 'keydown', function (event) {
      if (event.keyCode === 13) {
        quill.focus();
      }
    });
}

ImageManager.prototype.hideOperates = function () {
  $('.plugin-image-success-buttons').css('display', 'none');
};

ImageManager.prototype.getHTMLByTemplate = function (template, data) {
  return template.replace(/\{\{([^\{\}]+)\}\}/g, function (string, key1) {
    return data[key1];
  });
};

ImageManager.prototype.createPluginImageBlot = function (options, template) {
  var imageManager = this;
  var Block = Quill.import('blots/block');
  class PluginImageBlot extends Block {
    constructor(node) {
      super(node);
      var $node = $(node);
      var token = $node.data('token');
      if (token) {
        imageManager.images[token].$node = $node;
      }
      var imageContext = imageManager.images[token].imageContext;
      var error = imageContext.error || {};
      $node.html(
        imageManager.getHTMLByTemplate(
          template,
          Object.myAssign({}, imageContext, {
            url: $node.data('src'),
            desc: $node.data('desc'),
            errorMessage: error.message || ' ',
            display:
              error.code < -1000 && error.code > -1100 ? 'block' : 'none',
          })
        )
      );
    }
    static create(value) {
      var node = super.create();
      var $node = $(node);
      $node.data('src', value.src);
      $node.data('desc', value.desc);
      $node.data('token', value.token);
      $node.attr('contenteditable', value.contenteditable);
      return node;
    }
    static formats(node) {
      var $node = $(node);
      return {
        src: $node.data('src'),
        desc: $node.data('desc'),
        token: $node.data('token'),
        contenteditable: $node.attr('contenteditable'),
      };
    }
  }
  Object.myAssign(PluginImageBlot, options);
  Quill.register(PluginImageBlot);
  Block.allowedChildren.push(PluginImageBlot);
};

ImageManager.prototype.get = function (token) {
  return this.images[token];
};

ImageManager.prototype.add = function (imageContext) {
  var status = imageContext.status;
  var token = imageContext.token;
  var url = imageContext.url;
  var desc = imageContext.desc || '';
  var className = this.imageContextConfig[status].className;
  this.images[token] = {
    $node: null,
    imageContext,
  };
  var range = quill.getSelection() || {
    index: this.MPEditor.lastSelection || 0,
  };
  quill.insertText(range.index, '\n');
  quill.clipboard.dangerouslyPasteHTML(
    range.index + 1,
    `<div class="${className}" data-token="${token}" data-src="${url}" data-desc="${desc}" contenteditable="false"><br></div>`
  );
};

ImageManager.prototype.statusChange = function (imageContext, callback) {
  if (callback === undefined) {
    callback = Function.prototype;
  }
  var imageManager = this;
  var status = imageContext.status;
  var token = imageContext.token;
  var url = imageContext.url;
  var error = imageContext.error || {};
  var className = this.imageContextConfig[status].className;
  var template = this.imageContextConfig[status].template;
  var $node = this.images[token].$node;
  var desc = this.images[token].desc || '';
  if (status === 0) {
    this.images[token].desc = $node.find('.img-desc').text();
  }
  if ($node) {
    if (url) {
      var image = new Image();
      image.src = url;
      image.onload = function () {
        image.onload = null;
        $node.replaceWith(
          $(
            `<div class="${className}" data-token="${token}" data-src="${url}" data-desc="${desc}" contenteditable="false"><br></div>`
          )
        );
        callback();
      };
      image.onerror = function () {
        image.onerror = null;
        imageManager.statusChange(
          Object.myAssign(imageContext, {
            status: 1,
            error: {
              code: -1001,
              message: '网络出错',
            },
          }),
          callback
        );
      };
    } else {
      $node.replaceWith(
        $(
          `<div class="${className}" data-token="${token}" data-src="${url}" data-desc="${desc}" contenteditable="false"><br></div>`
        )
      );
      callback();
    }
  }
};

ImageManager.prototype.getImageSize = function (url) {
  return new Promise(function (resolve, reject) {
    var image = new Image();
    image.onload = function () {
      image.onload = null;
      resolve({
        width: image.width,
        height: image.height,
      });
    };
    image.onerror = function () {
      image.onerror = null;
      resolve({
        width: 0,
        height: 0,
      });
    };
    image.src = url;
  });
};
