'use strict';
function OperateButton(id, identifier) {
  this.id = id;
  this.identifier = identifier;
  this.active = false;
  this.button = $(identifier);
  this.active = false;
  this.lastSelection = 0;
}

OperateButton.prototype.trigger = function() {
  this.button.click();
};

function EventManager() {
  this.events = {};
}

EventManager.prototype.add = function(options, callbackMap) {
  var token = String(Date.now());
  this.events[token] = callbackMap;
  SHEditor.triggerCallDialog(Object.myAssign({}, options, { token }));
};

EventManager.prototype.trigger = function(token, type) {
  var callbackMap = this.events[token];
  var callback = callbackMap[type];
  if (callback) {
    callback();
  }
  delete this.events[token];
};

function EventManager() {
  this.events = {};
}

EventManager.prototype.add = function(options, callbackMap) {
  var token = String(Date.now());
  this.events[token] = callbackMap;
  SHEditor.triggerCallDialog(Object.myAssign({}, options, { token }));
};

EventManager.prototype.trigger = function(token, type) {
  var callbackMap = this.events[token];
  var callback = callbackMap[type];
  if (callback) {
    callback();
  }
  delete this.events[token];
};

function MPEditorClass() {
  this.correctedTagStatus = { disabled: false };
  this.env = /android/i.test(window.navigator.userAgent) ? 'Android' : 'iOS';
  this.operates = {};
  var identifierMap = {
    bold: '.ql-bold',
    heading: '.ql-header',
    quote: '.ql-blockquote',
    unorderedList: '.ql-list[value="bullet"]',
    orderedList: '.ql-list[value="ordered"]',
  };
  for (var id in identifierMap) {
    this.operates[id] = new OperateButton(id, identifierMap[id]);
  }
  this.imageManager = new ImageManager(this);
  this.contentManager = new ContentManager($('#title'), $('.ql-editor'), this);
  this.eventManager = new EventManager();

  this.showSection = this.debounce(function() {
    var contentManager = this.contentManager;
    var imageManager = this.imageManager;
    if (contentManager.loadingContent) {
      return;
    }
    var textHeight = contentManager.textHeight;
    var fixHeight = contentManager.fixHeight;
    var $content = contentManager.$content;
    var editorDOM = $content[0];
    var clientHeight =
      this.env === 'Android' ? document.documentElement.clientHeight : 247;
    var selection = quill.getSelection();
    if (!selection || imageManager.descFocused) {
      return;
    }
    var top = quill.getBounds(selection.index).top;
    var editorTop = editorDOM.getBoundingClientRect().top;
    var realTop = editorTop + top;
    var realBottom = clientHeight - realTop - textHeight;
    var scrollObject = {
      left: 0,
      behavior: 'smooth',
    };
    if (realTop > fixHeight && realBottom < fixHeight) {
      scrollObject.top = fixHeight - realBottom;
      window.scrollBy(scrollObject.left, scrollObject.top);
    } else if (realTop < fixHeight && realBottom > fixHeight) {
      scrollObject.top = realTop - fixHeight;
      window.scrollBy(scrollObject.left, scrollObject.top);
    }
  });

  // 初始化
  var MPEditor = this;
  var contentManager = MPEditor.contentManager;
  var imageManager = MPEditor.imageManager;
  var eventManager = MPEditor.eventManager;

  SHEditor.triggerWebviewJSReady();

  // 清除粘贴样式
  quill.clipboard.addMatcher(Node.ELEMENT_NODE, function(node, delta) {
    delta.ops.map(function(item) {
      if (
        !$(node).hasClass('plugin-image-success') &&
        !$(node).hasClass('plugin-image-fail') &&
        !$(node).hasClass('plugin-image-uploading') &&
        !$(node).hasClass('plugin-video') &&
        !$(node).is('mp-plugin')
      ) {
        delete item.attributes;
      }
    });
    return delta;
  });
  // 禁止粘贴plugin-row
  quill.clipboard.addMatcher('DIV', function(node, delta) {
    if (node.className.indexOf('plugin-row') > -1) {
      delta.ops.map(function(item) {
        delete item.attributes;
      });
    }
    return delta;
  });
  // 禁止粘贴a
  quill.clipboard.addMatcher('A', function(node, delta) {
    delta.ops.map(function(item) {
      delete item.attributes;
    });
    return delta;
  });
  // 禁止粘贴h2、h3、h4、h5、h6
  function handleH(node, delta) {
    var del = delta && delta.ops && delta.ops[0] && delta.ops[0].attributes;
    if (del) delete del.header;
    return new Delta().insert(node.innerText + '\n', del);
  }
  quill.clipboard.addMatcher('H2', handleH);
  quill.clipboard.addMatcher('H3', handleH);
  quill.clipboard.addMatcher('H4', handleH);
  quill.clipboard.addMatcher('H5', handleH);
  quill.clipboard.addMatcher('H6', handleH);

  quill.on('text-change', function() {
    MPEditor.showSection();
    SHEditor.updateTextCount(contentManager.getTextCount());
  });

  quill.on('editor-change', function(delta, oldDelta, source) {
    var selection = quill.getSelection();
    if (selection) {
      MPEditor.lastSelection = selection.index;
    }
    setTimeout(MPEditor.correctTagStatus.bind(MPEditor), 0);
  });

  // 键盘弹起
  window.addEventListener('resize', MPEditor.showSection);

  // // 模拟初始化html
  // setTimeout(function() {
  //   return MPEditor.getDisplayContent(
  //     `<p><embed width="500" height="300" bid="203774948" wmode="Transparent" allowfullscreen="true" allowscriptaccess="always" src="//share.vrs.sohu.com/my/v.swf&amp;topBar=1&amp;id=203774948&amp;autoplay=false&amp;from=page" type="application/x-shockwave-flash"></p><mp-plugin data-type="34" data-info="[{&quot;cardUuid&quot;:559}]"></mp-plugin><blockquote>爱神的箭啊哈斯柯达哈萨克大家好has电话卡交互啥的卡就会三大框架啊阿萨德哈克斯静待花开</blockquote><p class="ql-align-center"><img src="https://statics.itc.cn/mp-new/card39.png" max-width="600"><span class="img-desc" style="font-size: 16px;"></span></p>`
  //   ).then(function(realHTML) {
  //     $('.ql-editor')
  //       .html(realHTML)
  //       .append('<p></br></p>')[0]
  //       .focus();
  //   });
  // }, 1000);

  // 传给客户端html
  SHEditor.getRichTextContent(function(setHtmlString) {
    setHtmlString({
      title: contentManager.$title.val(),
      content: contentManager.getUploadContent(),
    });
  });

  // 获取客户端html
  SHEditor.onRichTextContentUpdate(function(object) {
    var title = object.title;
    var content = object.content;
    if (title) {
      contentManager.setTitle(title);
    }
    if (content) {
      MPEditor.getDisplayContent(content).then(function(realHTML) {
        console.log(realHTML);
        contentManager.setContent(realHTML);
        setTimeout(function() {
          SHEditor.triggerContentInitiated({
            content: contentManager.getUploadContent(),
          });
        }, 0);
      });
    }
  });

  // 处理弹窗回调
  SHEditor.onDialogButtonClicked(function(object) {
    var token = object.token;
    var type = object.type;
    eventManager.trigger(token, type);
  });

  // 插入图片
  SHEditor.onImageInsert(function(imageContext) {
    imageManager.add(imageContext);
    imageContext.onStatusChange = function() {
      imageManager.statusChange(imageContext, function() {
        MPEditor.showSection();
      });
    };
  });

  // 其他按钮
  MPEditor.correctTagStatus();
  SHEditor.onTextTagChanged(function(tag) {
    MPEditor.operates[tag].trigger();
    MPEditor.correctedTagStatus.disabled = false;
    MPEditor.correctTagStatus();
  });

  var Block = Quill.import('blots/block');
  var imageContextConfig = imageManager.imageContextConfig;
  for (var status in imageContextConfig) {
    var blotName = imageContextConfig[status].blotName;
    var tagName = imageContextConfig[status].tagName;
    var className = imageContextConfig[status].className;
    var template = imageContextConfig[status].template;
    imageManager.createPluginImageBlot(
      {
        blotName,
        tagName,
        className,
      },
      template
    );
  }
  class MpPluginBlot extends Block {
    constructor(node) {
      super(node);
      node.innerHTML = '[推广卡片]';
    }
    static create() {
      var node = super.create();
      node.innerHTML = '[推广卡片]';
      node.setAttribute('contenteditable', false);
      return node;
    }
  }
  MpPluginBlot.tagName = 'mp-plugin';
  MpPluginBlot.blotName = 'mp-plugin';
  Quill.register(MpPluginBlot);
  Block.allowedChildren.push(MpPluginBlot);

  class PluginVideoBlot extends Block {
    constructor(node) {
      super(node);
      node.innerHTML = '[视频]';
    }
    static create() {
      var node = super.create();
      node.innerHTML = '[视频]';
      node.setAttribute('contenteditable', false);
      return node;
    }
  }
  PluginVideoBlot.tagName = 'p';
  PluginVideoBlot.className = 'plugin-video';
  PluginVideoBlot.blotName = 'plugin-video';
  Quill.register(PluginVideoBlot);
}

MPEditorClass.prototype.correctTagStatus = function(dispatch) {
  if (dispatch === undefined) {
    dispatch = true;
  }
  var operates = this.operates;
  for (var id in operates) {
    var operate = operates[id];
    operate.active = operate.button.hasClass('ql-active');
    this.correctedTagStatus[id] = operate.active;
  }
  if (dispatch) {
    SHEditor.triggerTextTagChange(this.correctedTagStatus);
  }
};

MPEditorClass.prototype.debounce = function(fn, delay) {
  if (delay === undefined) {
    delay = 150;
  }
  var timer = null;
  var MPEditor = this;
  return function() {
    timer && clearTimeout(timer);
    timer = setTimeout(function() {
      fn.apply(MPEditor, arguments);
    }, delay);
  };
};

// 获取字符串长度
MPEditorClass.prototype.computeWordCount = function(string, limit) {
  var length = 0;
  var realLength = 0;
  var res = '';
  string = (string || '').replace(
    /\uD83C[\uDF00-\uDFFF]|\uD83D[\uDC00-\uDE4F]/g,
    ''
  );
  string.split('').forEach(function(value) {
    value.match(/[^\x00-\xff]/gi) != null ? (length += 1) : (length += 0.5);
    if (limit !== undefined) {
      if (limit >= length) {
        res += value;
        realLength = length;
      }
    } else {
      realLength = length;
    }
  });
  return {
    string: limit === undefined ? string : res,
    length: Math.floor(realLength),
  };
};

MPEditorClass.prototype.getDisplayContent = function(content) {
  var MPEditor = this;
  var index = 0;
  var images = [];
  content = content
    .replace(/>(\s+)</g, '><')
    .replace(/<mp-plugin(.*?)>(.*?<\/mp-plugin>)/gi, function(
      string,
      key1,
      key2
    ) {
      index++;
      if (index === 1) {
        return `<p></p><mp-plugin${key1} contenteditable="false">${key2}<p></p>`;
      }
      return `<mp-plugin${args[1]} contenteditable="false">${args[2]}<p></p>`;
    })
    .replace(/<p[^>]*><embed(.*?)><\/p>/gi, function(string, key1) {
      index++;
      if (index === 1) {
        return `<p></p><p class="plugin-video" contenteditable="false"${key1}></p><p></p>`;
      }
      return `<p class="plugin-video" contenteditable="false"${key1}></p><p></p>`;
    })
    .replace(
      /<p class="ql-align-center"><img.*?src="(.*?)".*?><span.*?class="img-desc".*?>(.*?)<\/span><\/p>/g,
      function(string, key1, key2) {
        var url = key1;
        var desc = key2;
        return `<p></p><div class="plugin-image-success" data-src="${url}" data-desc="${desc}" data-token="" contenteditable="false"><br></div>`;
      }
    )
    .replace(
      /<p class="ql-align-center"><img.*?src="(.*?)".*?><\/p>/g,
      function(string, key1) {
        var url = key1;
        return `<p></p><div class="plugin-image-success" data-src="${url}" data-desc="" data-token="" contenteditable="false"><br></div>`;
      }
    )
    .replace(
      /<div class="plugin-image-success" data-src="(.*?)".*?<\/div>/g,
      function(string, key1) {
        images.push(key1);
        return string;
      }
    );
  return Promise.all(
    images.map(function(image) {
      return MPEditor.imageManager.getImageSize(image).then(function(size) {
        return {
          url: image,
          size,
        };
      });
    })
  ).then(function(configs) {
    var tokens = configs.map(function(config) {
      var imageManager = MPEditor.imageManager;
      var url = config.url;
      var size = config.size;
      var imageContext = SHEditor.createImageContext({
        url,
        size,
      });
      var token = imageContext.token;
      imageManager.add(imageContext);
      imageContext.onStatusChange = function() {
        imageManager.statusChange(imageContext, function() {
          MPEditor.showSection();
        });
      };
      return token;
    });
    return content.replace(
      /(<div class="plugin-image-success" data-src=".*?" data-desc=".*?" data-token=")(" contenteditable="false"><br><\/div>)/g,
      function(string, key1, key2) {
        var token = tokens.shift();
        return key1 + token + key2;
      }
    );
  });
};

window.MPEditor = new MPEditorClass();
